@@ -10,6 +10,9 @@ import requests
 import json
 import georouting.utils as gtl
 import folium
+import networkx as nx
+import osmnx as ox
+
 
 class GoogleRoute:
     """
@@ -269,13 +272,13 @@ class EsriRoute:
 
     The class has the following methods:
 
-    get_duration: returns the total travel time of the route.
+    - `get_duration`: returns the total travel time of the route.
 
-    get_distance: returns the total distance of the route in meters.
+    - `get_distance`: returns the total distance of the route in meters.
 
-    get_route: returns the entire route as a dictionary.
+    - `get_route`: returns the entire route as a dictionary.
 
-    get_route_geopandas: raises a NotImplementedError. This method is not yet implemented and will be added in the future.
+    - `get_route_geopandas`: raises a NotImplementedError. This method is not yet implemented and will be added in the future.
 
 
     """
@@ -284,18 +287,78 @@ class EsriRoute:
         self.route = route
 
     def get_duration(self):
-        return self.route["routes"]["features"][0]["attributes"]["Total_TravelTime"]
+        return (
+            self.route["routes"]["features"][0]["attributes"]["Total_TravelTime"] * 60
+        )
 
     def get_distance(self):
         return (
-            self.route["routes"]["features"][0]["attributes"]["Total_Miles"] * 1609.344
+            self.route["routes"]["features"][0]["attributes"]["Total_Kilometers"] * 1000
         )
 
     def get_route(self):
         return self.route
 
     def get_route_geopandas(self):
-        raise NotImplementedError
+        linestring = self.route["routes"]["features"][0]["geometry"]["paths"][0]
+        gdf = gpd.GeoDataFrame(
+            geometry=gpd.GeoSeries(sg.LineString(linestring)), crs="EPSG:4326"
+        )
+
+        gdf["speed (m/s)"] = self.get_distance() / self.get_duration()
+
+        return gdf
+
+
+class OSMNXRoute:
+    def __init__(self, route):
+        self.route = route[0]
+        self.G = route[1]
+
+    def _get_durations(self):
+        durations = ox.utils_graph.get_route_edge_attributes(
+            self.G, self.route, "travel_time"
+        )
+        return durations
+
+    def _get_distances(self):
+        distances = ox.utils_graph.get_route_edge_attributes(
+            self.G, self.route, "length"
+        )
+        return distances
+
+    def get_duration(self):
+        durations = self._get_durations()
+
+        return round(sum(durations))
+
+    def get_distance(self):
+        edge_lengths = self._get_distances()
+
+        return round(sum(edge_lengths))
+
+    def get_route(self):
+        return self.route
+
+    def get_route_geopandas(self):
+        subgraph = self.G.subgraph(self.route)
+        gdf_nodes, gdf_edges = ox.graph_to_gdfs(subgraph)
+
+        # gdf_edges["duration (s)"] = self._get_durations()
+        # gdf_edges["distance (m)"] = self._get_distances()
+        # gdf_edges["speed (m/s)"] = gdf_edges["distance (m)"] / gdf_edges["duration (s)"]
+
+        gdf_edges.rename(
+            columns={"length": "distance (m)", "travel_time": "duration (s)"},
+            inplace=True,
+        )
+        gdf_edges["speed (m/s)"] = gdf_edges["distance (m)"] / gdf_edges["duration (s)"]
+
+        gdf_edges = gdf_edges[
+            ["duration (s)", "distance (m)", "geometry", "speed (m/s)"]
+        ]
+        gdf_edges = gdf_edges.reset_index(drop=True)
+        return gdf_edges
 
 
 class MapboxRoute:
@@ -367,7 +430,7 @@ class Route(object):
 
     """
 
-    def __init__(self, route,origin, destination):
+    def __init__(self, route, origin, destination):
         """
         Initialize a Route object by passing the routing engine's route object.
 
@@ -400,16 +463,24 @@ class Route(object):
         Get the route information as a GeoDataFrame.
         """
         return self.route.get_route_geopandas()
+
     def plot_route(self):
         """
         Plot the route on a map.
         """
         gdf = self.get_route_geopandas()
-        m = gdf.explore(column="speed (m/s)",style_kwds={"weight":11,"opacity":0.8})
+        m = gdf.explore(
+            column="speed (m/s)",
+            style_kwds={"weight": 11, "opacity": 0.8},
+            cmap="RdYlGn",
+        )
         # add a red destination marker, don't show i in the map
-        folium.Marker([self.destination[0],self.destination[1]],
-        icon=folium.Icon(color="red",icon_color="white",icon="circle", prefix="fa")
-            ).add_to(m)
+        folium.Marker(
+            [self.destination[0], self.destination[1]],
+            icon=folium.Icon(
+                color="red", icon_color="white", icon="circle", prefix="fa"
+            ),
+        ).add_to(m)
 
         # folium.Marker([one_od_pair["AHA_ID_lat"],one_od_pair["AHA_ID_lon"]],
         # icon=folium.Icon(color="red",icon_color="white",icon="circle", prefix="fa")).add_to(m)
@@ -437,18 +508,20 @@ class BaseRouter(object):
                 items.append(item)
         od_matrix = pd.DataFrame(
             items,
-            columns=["orgin_lat", "orgin_lon", "destination_lat", "destination_lon"],
+            columns=["origin_lat", "origin_lon", "destination_lat", "destination_lon"],
         )
 
         return od_matrix
 
-    def get_route(self, origin, destination):
-        return Route(self._get_request(origin, destination))
+    # def get_route(self, origin, destination):
+    #     return Route(self._get_request(origin, destination))
 
-    def get_distances_batch(self, origins, destinations, max_batch_size=25, append_od=False):
+    def get_distances_batch(
+        self, origins, destinations, max_batch_size=25, append_od=False
+    ):
         """
-        This method returns a Pandas dataframe contains duration and disatnce for all the `origins` and `destinations` pairs. Use this function if you don't want to get duration and distance for all possible combinations between each origin and each destination. 
-        
+        This method returns a Pandas dataframe contains duration and disatnce for all the `origins` and `destinations` pairs. Use this function if you don't want to get duration and distance for all possible combinations between each origin and each destination.
+
         The origins and destinations parameters are lists of origin-destination pairs. They should be the same length.
 
         If the `append_od` parameter is set to True, the method also returns the input origin-destination pairs.
@@ -457,21 +530,23 @@ class BaseRouter(object):
         # convert the origins and destinations to lists
         origins = gtl.convert_to_list(origins)
         destinations = gtl.convert_to_list(destinations)
-        
+
         # check if the origins and destinations are the same length
         if len(origins) != len(destinations):
-            raise ValueError("The origins and destinations should have the same length.")
-        
+            raise ValueError(
+                "The origins and destinations should have the same length."
+            )
+
         # divide the origins and destinations into batches
 
         batches = gtl.get_batch_od_pairs(origins, destinations, max_batch_size)
 
-        # get the distance matrix for each batch 
+        # get the distance matrix for each batch
         results = []
         for batch in batches:
             res = self.get_distance_matrix(batch[0], batch[1])
             results.append(res)
-        
+
         # concatenate the results
         df = pd.concat(results, axis=0)
         # revert the order of the rows
@@ -481,14 +556,22 @@ class BaseRouter(object):
             # convert the origins and destinations to numpy arrays
             origins = np.array(origins)
             destinations = np.array(destinations)
-            df["origin_lat"] = origins[:,0]
-            df["origin_lon"] = origins[:,1]
-            df["destination_lat"] = destinations[:,0]
-            df["destination_lon"] = destinations[:,1]  
+            df["origin_lat"] = origins[:, 0]
+            df["origin_lon"] = origins[:, 1]
+            df["destination_lat"] = destinations[:, 0]
+            df["destination_lon"] = destinations[:, 1]
+
+            df = df[
+                [
+                    "origin_lat",
+                    "origin_lon",
+                    "destination_lat",
+                    "destination_lon",
+                    "distance (m)",
+                    "duration (s)",
+                ]
+            ]
 
-            df = df[["origin_lat", "origin_lon", "destination_lat", "destination_lon", 
-            "distance (m)", "duration (s)"]] 
-        
         return df
 
 
